#' @title Miscellaneous Functions for "Grid" Graphics
#' @description Provides a number of user-level functions to work with "grid" graphics, notably to arrange multiple grid-based plots on a page, and draw tables.
#' @name gridExtra-package
#' @aliases gridExtra
#' @docType package
#' @author baptiste Auguie \email{baptiste.auguie@@gmail.com}
#' @import gtable 
#' @import grid 
#' @references
#' R Graphics by Paul Murrell (Chapman & Hall/CRC, August 2005)
#' @keywords packagelibrary
#' @seealso \code{\link{Grid}}
#' 
function()
  NULL
